from rembed.core import consumer

from hamcrest import assert_that, equal_to
from mock import patch, Mock

import pytest


def test_should_discover_and_get_oembed_url():
    with patch('rembed.core.discovery.get_oembed_url') as mock_get_url, \
        patch('rembed.core.parse.parse_oembed') as mock_parse, \
            patch('requests.get') as mock_get:

        mock_get_url.return_value = (
            'json', 'http://example.com/oembed?format=json')

        response = Mock()
        response.ok = True
        response.text = 'hello, world'
        mock_get.return_value = response

        parsed = Mock()
        mock_parse.return_value = parsed

        assert_that(consumer.get_oembed_response(
            'http://example.com/'), equal_to(parsed))

        mock_get_url.assert_called_with(
            'http://example.com/', max_width=None, max_height=None)
        mock_get.assert_called_with('http://example.com/oembed?format=json')
        mock_parse.assert_called_with('json', 'hello, world')


def test_should_discover_and_get_oembed_url_with_max_width_and_height():
    with patch('rembed.core.discovery.get_oembed_url') as mock_get_url, \
        patch('rembed.core.parse.parse_oembed') as mock_parse, \
            patch('requests.get') as mock_get:

        mock_get_url.return_value = (
            'json',
            'http://example.com/oembed?format=json&maxwidth=100&maxheight=200')

        response = Mock()
        response.ok = True
        response.text = 'hello, world'
        mock_get.return_value = response

        parsed = Mock()
        mock_parse.return_value = parsed

        assert_that(consumer.get_oembed_response(
            'http://example.com/', 100, 200), equal_to(parsed))

        mock_get_url.assert_called_with(
            'http://example.com/', max_width=100, max_height=200)
        mock_get.assert_called_with(
            'http://example.com/oembed?format=json&maxwidth=100&maxheight=200')
        mock_parse.assert_called_with('json', 'hello, world')


def test_should_raise_error_on_request_error():
    with patch('rembed.core.discovery.get_oembed_url') as mock_get_url, \
        patch('requests.get') as mock_get, \
            pytest.raises(consumer.REmbedConsumerError):

        mock_get_url.return_value = (
            'json', 'http://example.com/oembed?format=json')

        response = Mock()
        response.ok = False
        response.text = 'hello, world'
        mock_get.return_value = response

        consumer.get_oembed_response('http://example.com/')


def test_should_embed():
    with patch('rembed.core.consumer.get_oembed_response') as mock_get, \
            patch('rembed.core.render.render_response') as mock_render:
        response = Mock()
        response.title = 'hello'

        mock_get.return_value = response
        mock_render.return_value = '<h1>hello</h1>'

        result = consumer.embed('http://example.com/')
        assert_that(result, equal_to('<h1>hello</h1>'))

        mock_get.assert_called_with('http://example.com/', None, None)
        mock_render.assert_called_with('http://example.com/', response, None)


def test_should_embed_with_max_width_and_height():
    with patch('rembed.core.consumer.get_oembed_response') as mock_get, \
            patch('rembed.core.render.render_response') as mock_render:
        response = Mock()
        response.title = 'hello'

        mock_get.return_value = response
        mock_render.return_value = '<h1>hello</h1>'

        result = consumer.embed('http://example.com/', 100, 200)
        assert_that(result, equal_to('<h1>hello</h1>'))

        mock_get.assert_called_with('http://example.com/', 100, 200)
        mock_render.assert_called_with('http://example.com/', response, None)


def test_should_embed_with_custom_template_dir():
    with patch('rembed.core.consumer.get_oembed_response') as mock_get, \
            patch('rembed.core.render.render_response') as mock_render:
        response = Mock()
        response.title = 'hello'

        mock_get.return_value = response
        mock_render.return_value = '<h1>hello</h1>'

        result = consumer.embed(
            'http://example.com/', template_dir='templates')
        assert_that(result, equal_to('<h1>hello</h1>'))

        mock_get.assert_called_with('http://example.com/', None, None)
        mock_render.assert_called_with(
            'http://example.com/', response, 'templates')
